#!/usr/bin/perl

use Getopt::Long;
use vars qw($opt_h $PROGNAME $opt_w $opt_c $opt_t $opt_type $opt_vi $msg $state);
use lib "/usr/lib/nagios/plugins";
use lib "/opt/nagios/libexec";
use utils qw(%ERRORS &print_revision &support &usage );

my $os=`uname -s`;

sub print_help;
sub print_usage;
sub process_arguments;

Getopt::Long::Configure('bundling');
$status=process_arguments( $os);
if ($status){
	print "ERROR:  Processing Arguments\n";
	exit $ERRORS{'WARNING'};
	}
	
$SIG{'ALRM'} = sub {
        print ("ERROR: timed out waiting for $PROGNAME \n");
	        exit $ERRORS{"WARNING"};
};
alarm($opt_t);

my ($usedhandles, $freehandles, $file_max) = ( 0, 0, -1);
if( $os =~ /^Linux$/) {
	my ($nb_allochandles, $nb_freehandles);
	$procfile="/proc/sys/fs/file-nr";
	open(FILE, $procfile) or die "$procfile not exits!";
	my $line=<FILE>;
	close FILE;
	($nb_allochandles, $nb_freehandles, $file_max)= split(/\s/, $line);
	### See /usr/src/linux/Documentation/sysctl/fs.txt : the real number of used files is
	# the first field minus the second one
	$usedhandles = $nb_allochandles - $nb_freehandles;
	$freehandles = $file_max - $usedhandles;
}
elsif( $os =~ /^SunOS$/) {
	$usedhandles=`/usr/bin/ls -l /proc/*/fd | /usr/bin/egrep -v "^$|^/|^total" | /usr/bin/wc -l`
}
elsif( $os =~ /^AIX$/) {
	my $output = `/opt/nagios/libexec/count_open_files`;
	if( $output =~ /^Found a total of (\d+) open file descriptors$/) {
		$usedhandles=$1;
	}
	else {
        print( "ERROR: Unable to determine number of open file descriptors.\n");
        exit $ERRORS{"WARNING"};
	}
}

my( $warning_threshold, $cricital_threshold);

if( $opt_type =~ /pf/) {
	my $freepercent = (( $freehandles / $file_max) * 100);
	if ($freepercent <= $opt_c ) {
		$msg = sprintf( "CRITICAL: free files handles (%.1f%%) is below critical threshold (threshold=%.1f%%/%.1f%%", $freepercent, $opt_w, $opt_c);
		$state=$ERRORS{'CRITICAL'};
	}
	elsif ($freepercent <= $opt_w) {
		$msg = sprintf( "WARNING: free file handles (%.1f%%) is below warning threshold (threshold=%.1f%%/%.1f%%)", $freepercent, $opt_w, $opt_c);
		$state=$ERRORS{'WARNING'};
	}
	else {
		$msg = sprintf( "OK: free file handles (%.1f%%) is above threshold (%.1f%%/%.1f%%)", $freepercent, $opt_w, $opt_c);
		$state=$ERRORS{'OK'};
	}
	printf( "%s|percent_free_handles=%.1f;%.1f;%.1f\n", $msg, $freepercent, $opt_w, $opt_c);
}
elsif( $opt_type =~ /pu/) {
	my $usedpercent = (( $usedhandles / $file_max) * 100);
	if ($usedpercent >= $opt_c ) {
		$msg = sprintf( "CRITICAL: used files handles (%.1f%%) exceeds critical threshold (threshold=%.1f%%/%.1f%%)", $usedpercent, $opt_w, $opt_c);
		$state=$ERRORS{'CRITICAL'};
	}
	elsif ($usedpercent >= $opt_w) {
		$msg = sprintf( "WARNING: used file handles (%.1f%%) exceeds warning threshold (threshold=%.1f%%/%.1f%%)", $usedpercent, $opt_w, $opt_c);
		$state=$ERRORS{'WARNING'};
	}
	else {
		$msg = sprintf( "OK: used file handles (%.1f%%) is below threshold (%.1f%%/%.1f%%)", $usedpercent, $opt_w, $opt_c);
		$state=$ERRORS{'OK'};
	}
	printf( "%s|percent_used_handles=%.1f;%.1f;%.1f\n", $msg, $usedpercent, $opt_w, $opt_c);
}
elsif( $opt_type =~ /nf/) {
	if ($freehandles <= $opt_c ) {
		$msg = sprintf( "CRITICAL: free files handles (%d) is below critical threshold (threshold=%d/%d)", $freehandles, $opt_w, $opt_c);
		$state=$ERRORS{'CRITICAL'};
	}
	elsif ($freehandles <= $opt_w){
		$msg = sprintf( "WARNING: free file handles (%d) is below warning threshold (threshold=%d/%d)", $freehandles, $opt_w, $opt_c);
		$state=$ERRORS{'WARNING'};
	}
	else {
		$msg = sprintf( "OK: free file handles (%d) is above threshold (%d/%d)", $freehandles, $opt_w, $opt_c);
		$state=$ERRORS{'OK'};
	}
	printf( "%s|free_handles=%d;%d;%d\n", $msg, $freehandles, $opt_w, $opt_c);
}
elsif( $opt_type =~ /nu/) {
	if ($usedhandles >= $opt_c ){
		$msg = sprintf( "CRITICAL: used files handles (%d) exceeds critical threshold (threshold=%d/%d)", $usedhandles, $opt_w, $opt_c);
		$state=$ERRORS{'CRITICAL'};
	}
	elsif ($usedhandles >= $opt_w && $usedhandles < $opt_c){
		$msg = sprintf( "WARNING: used file handles (%d) exceeds warning threshold (threshold=%d/%d)", $usedhandles, $opt_w, $opt_c);
		$state=$ERRORS{'WARNING'};
	}
	else {
		$msg = sprintf( "OK: used file handles (%d) is below threshold (%d/%d)", $usedhandles, $opt_w, $opt_c);
		$state=$ERRORS{'OK'};
	}
	printf( "%s|used_handles=%d;%d;%d\n", $msg, $usedhandles, $opt_w, $opt_c);
}

exit $state;

########## SUBS ##############################################
sub process_arguments {
	my $os = shift;

	GetOptions ( 
			"w=s" => \$opt_w, "warning=s"  => \$opt_w,   # warning if above this number
            "c=s" => \$opt_c, "critical=s" => \$opt_c,   # critical if above this number
			"T=s" => \$opt_type, "type=s"  => \$opt_type,
			"t=i" => \$opt_t, "timeout=i"  => \$opt_t,
			"h"   => \$opt_h, "help"       => \$opt_h,
			"v"   => \$opt_v, "version"    => \$opt_v
		   );

	if ($opt_v){
		print_revision ($PROGNAME ,'$Revision: 1.1 $');
		exit $ERRORS{'OK'};
		}
	if ($opt_h){
		print_help( $os);
		exit $ERRORS{'OK'};
		}
	unless (defined $opt_t){
		$opt_t = $utils::TIMEOUT;
		}
	unless (defined $opt_type){
		$opt_type = 'pf';
		if( $os =~ /^(SunOS|AIX)$/) {
			$opt_type = 'nu';
			}
		}
	unless ($opt_type =~ /^pf$|^pu$|^nf$|^nu$/){
		print "Illegal type: $opt_type\n";
		print_usage( $os);
		exit $ERRORS{'UNKNOWN'};
		}
	if ($os =~ /^(SunOS|AIX)$/ && $opt_type !~ /^nu$/){
		print "Threshold type $opt_type not supported on this operating system.\n";
		print_usage( $os);
		exit $ERRORS{'UNKNOWN'};
		}
	unless (defined $opt_w && defined $opt_c){
		print_usage( $os);
		exit $ERRORS{'UNKNOWN'};
		}
	 if ( $opt_type =~ /pf|nf/ && $opt_w < $opt_c) {
                 print "Warning (-w) cannot be less than Critical (-c) for threshold type $opt_type!\n";
                 exit $ERRORS{'UNKNOWN'};
	         }
	 if ( $opt_type =~ /pu|nu/ && $opt_w > $opt_c) {
                 print "Warning (-w) cannot be greater than Critical (-c) for threshold type $opt_type!\n";
                 exit $ERRORS{'UNKNOWN'};
	         }
	return $ERRORS{'OK'};

}


sub print_usage {
	my $os = shift;

	if( $os =~ /^(SunOS|AIX)$/) {
        print "Usage: $PROGNAME -w <warn> -c <crit> [-t <timeout>] [-v version] [-h help]\n";
	}
	else {
        print "Usage: $PROGNAME -w <warn> -c <crit> [-T pf|pu|nf|nu] [-t <timeout>] [-v version] [-h help]\n";
	}
}


sub print_help {

	my $os = shift;

        print_revision($PROGNAME,'$Revision: 1.2 $');
        print "Copyright (c) 2007 Christophe Thiesset\n";
        print "\n";
        print_usage( $os);
        print "\n";
        print "   Checks the open files number against the max authorized or a specified number\n";
        print "-w (--warning)   = Percentage/number of opened files to generate warning alert\n";
        print "-c (--critical)  = Percentage/number of opened files to generate critical( w < c )\n";
		if( $os !~ /^(SunOS|AIX)$/) {
	        print "-T (--type)      = Type of thresholds: \n";
			print "      pf: thresholds are percentage of system maximum file handles free (default)\n";
			print "      pu: thresholds are percentage of system maximum file handles used\n";
			print "      nf: thresholds are number of system file handles free\n";
			print "      nu: thresholds are number of system file handles used\n";
		}
        print "-t (--timeout)   = Plugin timeout in seconds (default = $utils::TIMEOUT)\n";
        print "-h (--help)\n";
        print "-v (--version)\n";
        print "\n\n";
        support();
}

