#!/bin/bash

# Check Memory Usage via `free -mt`

########################
# DECLARATIONS
########################

PROGNAME=`basename $0`
REVISION=`echo '$Revision: 1.1 $' | sed -e 's/[^0-9.]//g'`
OS_MINOR=$(sw_vers -productVersion | sed 's/10.\([0-9]*\)..*/\1/')

DEBUG=0

exitstatus=0
result=""
perfdata=""
pctWarning=""
pctCritical=""
pctSwpWarning=""
pctSwpCritical=""
rawOutput=0

status[0]="OK: "
status[1]="WARNING: "
status[2]="CRITICAL: "
status[3]="UNKNOWN: "

########################
# FUNCTIONS
########################

print_usage() {
  echo "Usage: $PROGNAME [options]"
  echo "  e.g. $PROGNAME -w 75 -W 25 -c 95 -C 75"
  echo
  echo "Options:"
  echo -e "\t --help    | -h       print help"
  echo -e "\t --version | -V       print version"
  echo -e "\t --verbose | -v       be verbose (debug mode)"
  echo -e "\t --raw     | -r       Use MB instead of % for output data"
  echo -e "\t -w [int]             set warning value for physical RAM used %"
  echo -e "\t -c [int]             set critical value for physical RAM used %"
  echo -e "\t -W [int]             set warning value for swap used %"
  echo -e "\t -C [int]             set critical value for swap used %"
  echo
  echo
}

print_help() {
#  print_revision $PROGNAME $REVISION
  echo "${PROGNAME} Revision: ${REVISION}"
  echo
  echo "This plugin checks local memory usage using 'free -mt'"
  echo
  print_usage
  echo
# support
  exit 3
}

parse_options() {
# parse cmdline arguments
  (( DEBUG )) && echo "Parsing options $1 $2 $3 $4 $5 $6 $7 $8"
  if [ "$#" -gt 0 ]; then
    while [ "$#" -gt 0 ]; do
      case "$1" in
        '--help'|'-h')
          print_help
          exit 3
          ;;
        '--version'|'-V')
          #print_revision $PROGNAME $REVISION
          echo "${PROGNAME} Revision: ${REVISION}"
          exit 3
          ;;
        '--verbose'|'-v')
          DEBUG=1
          shift 1
          ;;
        '--raw'|'-r')
          rawOutput=1
          shift 1
          ;;
        '-c')
          pctCritical="$2"
          shift 2
          ;;
        '-w')
          pctWarning="$2"
          shift 2
          ;;
        '-C')
          pctSwpCritical="$2"
          shift 2
          ;;
        '-W')
          pctSwpWarning="$2"
          shift 2
          ;;
        *)
          echo "Unknown option!"
          print_usage
          exit 3
          ;;
      esac
    done
  fi
}


function intval {
# $1: String or numeric value
# Sample:  v=$(intval "123 t")

# Remove all non numeric characters
n="$(echo "$1" |  sed 's/[^0-9]//g')"
if [[ $1 == *"K"* ]]; then
    echo ${n:0}
elif [[ $1 == *"G"* ]]; then
    echo $(bc <<< "scale=2;$n*1024")
else
    echo "$n"
fi
 
 
}



########################
# MAIN
########################

parse_options $@


 
swpTotalRaw=`sysctl vm.swapusage | awk '{ print $4 }'`
swpTotal=`echo $swpTotalRaw | awk -F . '{ print $1 }'`
#echo "SWAPTOTALRAW: $swpTotalRaw"
#echo "SWAPTOTAL: $swpTotal"
swpUsedRaw=`sysctl vm.swapusage | awk '{ print $7 }'`
swpUsed=`echo $swpUsedRaw | awk -F . '{ print $1 }'`
#echo "SWAPUSEDRAW: $swpUsedRaw"
#echo "SWAPUSED: $swpUsed"
 
 
PHY_FREE_AWK_COLUMN=10
PHY_USED_AWK_COLUMN=8
if [ $OS_MINOR -gt 8 ]; then
  PHY_FREE_AWK_COLUMN=6
  PHY_USED_AWK_COLUMN=2
fi

phyFreeRaw=`top -l 1 | grep PhysMem: | awk -v col=$PHY_FREE_AWK_COLUMN '{ print $col }'`
phyFree=$(intval $phyFreeRaw)
#echo "RAWFREE: $phyFreeRaw"
#echo "FREE: $phyFree"
phyUsedRaw=`top -l 1 | grep PhysMem: | awk -v col=$PHY_USED_AWK_COLUMN '{ print $col }'`
phyUsed=$(intval $phyUsedRaw)
#echo "RAWUSED: $phyUsedRaw"
#echo "USED: $phyUsed"
phyTotal=`expr $phyFree + $phyUsed`
phyAllUsed=$phyUsed

pctPhyUsed=$(bc <<< "scale=2;$phyUsed*100/$phyTotal")
 (( DEBUG )) && echo "pctPhyUsed=$pctPhyUsed"
pctPhyAllUsed=$(bc <<< "scale=2;$phyAllUsed*100/$phyTotal")
 (( DEBUG )) && echo "pctPhyAllUsed=$pctPhyAllUsed"
if [ $swpTotal -eq 0 ]; then
  pctSwpUsed=0
else
  pctSwpUsed=$(bc <<< "scale=2;$swpUsed*100/$swpTotal")
fi
 (( DEBUG )) && echo "pctSwpUsed=$pctSwpUsed"
 (( DEBUG )) && echo "rawOutput=$rawOutput"
 (( DEBUG )) && echo "pctWarning=$pctWarning"
 (( DEBUG )) && echo "pctCritical=$pctCritical"
 (( DEBUG )) && echo "pctSwpWarning=$pctSwpWarning"
 (( DEBUG )) && echo "pctSwpCritical=$pctSwpCritical"

if [ -n "$pctWarning" ]; then
  warning=$(bc <<< "scale=0;$pctWarning * $phyTotal / 100")
  (( DEBUG )) && echo "warning=$warning"
  if [ $(bc <<< "$pctPhyUsed >= $pctWarning") -ne 0 ]; then
    exitstatus=1
  fi
fi

if [ -n "$pctSwpWarning" ]; then
  swpWarning=$(bc <<< "scale=0;$pctSwpWarning * $swpTotal / 100")
  (( DEBUG )) && echo "swpWarning=$swpWarning"
  if [ $(bc <<< "$pctSwpUsed >= $pctSwpWarning") -ne 0 ]; then
    exitstatus=1
  fi
fi

if [ -n "$pctCritical" ]; then
  critical=$(bc <<< "scale=0;$pctCritical * $phyTotal / 100")
  (( DEBUG )) && echo "critical=$critical"
  if [ $(bc <<< "$pctPhyUsed >= $pctCritical") -ne 0 ]; then
    exitstatus=2
  fi
fi

if [ -n "$pctSwpCritical" ]; then
  swpCritical=$(bc <<< "scale=0;$pctSwpCritical * $swpTotal / 100")
  (( DEBUG )) && echo "swpCritical=$swpCritical"
  if [ $(bc <<< "$pctSwpUsed >= $pctSwpCritical") -ne 0 ]; then
    exitstatus=2
  fi
fi

if [ $rawOutput -eq 1 ]; then
  result="Memory Usage - ${phyUsed}MB of ${phyTotal}MB RAM used, ${swpUsed}MB of ${swpTotal}MB Swap used"
  perfdata="phyUsed=${phyUsed};${warning};${critical};0;${phyTotal} swpUsed=${swpUsed};${swpWarning};${swpCritical};0;${swpTotal}"
else
  result="Memory Usage - ${pctPhyUsed}% RAM, ${pctSwpUsed}% Swap"
  perfdata="phyUsed=${pctPhyUsed}%;${pctWarning};${pctCritical};0;100  swpUsed=${pctSwpUsed}%;${pctSwpWarning};${pctSwpCritical};0;100"
fi

echo "${status[$exitstatus]}${result} | ${perfdata}"
exit $exitstatus

